/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.gluten.ras

trait PlanModel[T <: AnyRef] {
  // Trivial tree operations.
  def childrenOf(node: T): Seq[T]
  def withNewChildren(node: T, children: Seq[T]): T
  def hashCode(node: T): Int
  def equals(one: T, other: T): Boolean

  // Group operations.
  def newGroupLeaf(groupId: Int): GroupLeafBuilder[T]
  def isGroupLeaf(node: T): Boolean
  def getGroupId(node: T): Int
}

object PlanModel {
  implicit class PlanModelImplicits[T <: AnyRef](model: PlanModel[T]) {
    def isLeaf(node: T): Boolean = {
      model.childrenOf(node).isEmpty
    }
  }
}

trait GroupLeafBuilder[T <: AnyRef] {
  def id(): Int
  def build(): T
}
